@@ -16,6 +16,7 @@
 import hashlib
 import select
 import random
+import shutil
 import string
 
 import os
@@ -232,6 +233,19 @@ class process(StdoutableProcess):
                             s = self.pipe.stderr.readline()
                             _cacheerr.append(s)
                     if self.pipe.poll() is not None:
+                        s = " "
+                        while True:
+                            s = self.pipe.stdout.readline()
+                            if not s:
+                                break
+                            yield s
+                            self._cachedata.append(s)
+                        while True:
+                            s = self.pipe.stderr.readline()
+                            if not s:
+                                break
+                            yield s
+                            _cacheerr.append(s)
                         break
         except KeyboardInterrupt:
             self.kill()
@@ -797,10 +811,11 @@ def readLinesFile(filename):
         raise StopIteration
 
 
-def readFile(filename, tailbyte=None):
+def readFile(filename, tailbyte=None, headbyte=None):
     """
     Прочитать целый файл или вернуть пустую строку.
     tailbyte: прочитать только последнее указанное количество байт
+    headbyte: прочитать только первое указанное количество байт
     """
     try:
         if path.exists(filename):
@@ -809,7 +824,10 @@ def readFile(filename, tailbyte=None):
                     seeksize = max(0, os.stat(filename).st_size - tailbyte)
                     if seeksize:
                         f.seek(seeksize)
-                return f.read()
+                if headbyte:
+                    return f.read(headbyte)
+                else:
+                    return f.read()
     except (OSError, IOError):
         pass
     return ""
@@ -1103,7 +1121,7 @@ def copyWithPath(fn, dest, prefix=None):
     """
     Копировать файл в dst с созданием каталогов
 
-    dest: каталог для копирывания
+    dest: каталог для копирования
     fn: имя файла
     prefix: префикс исходного файла
 
@@ -1345,6 +1363,7 @@ class DiskSpace(object):
         else:
             raise DiskSpaceError(str(p.readerr()))
 
+
 def calculate_exclude(base_dn, exclude=(), include=()):
     """
     Вычисление исключений с использованием фильтров include
@@ -1356,30 +1375,93 @@ def calculate_exclude(base_dn, exclude=(), include=()):
     for exclude_dn in exclude:
         # ищем фильтры для текущего исключения
         f_include = [x for x in include
-                     if x == exclude_dn or x.startswith("%s/"%exclude_dn)]
+                     if x == exclude_dn or x.startswith("%s/" % exclude_dn)]
         # если фильтра нет - то исключение добавляется полностью
         if not f_include:
             yield exclude_dn
         else:
             # пребираем все файлы в указанно исключении
-            for root, dirs, files in os.walk(path.join(base_dn,exclude_dn)):
-                rel_root = root[len(base_dn)+1:]
+            for root, dirs, files in os.walk(path.join(base_dn, exclude_dn)):
+                rel_root = root[len(base_dn) + 1:]
                 # если файл указан в фильтре, пропускаем его
-                for fn in (path.join(rel_root,x) for x in files):
+                for fn in (path.join(rel_root, x) for x in files):
                     if fn not in f_include:
                         yield fn
                 filtered_dn = []
                 for dn in dirs:
-                    rel_dn = path.join(rel_root,dn)
+                    rel_dn = path.join(rel_root, dn)
                     # если директория не используется как часть фильтра - возвращем её
-                    if all(not x.startswith("%s/"%rel_dn) and x != rel_dn
+                    if all(not x.startswith("%s/" % rel_dn) and x != rel_dn
                            for x in f_include):
                         filtered_dn.append(dn)
                         yield rel_dn
                     elif rel_dn in f_include:
-                            filtered_dn.append(dn)
+                        filtered_dn.append(dn)
 
                 # удаляем из обрабатываемых дирекоторий не использующиеся в фильтре
                 # или указанную конечную директорию
                 for dn in filtered_dn:
                     dirs.remove(dn)
+
+
+def get_free_dirname(dn):
+    """
+    Возвращает имя указанной директорий если оно отсутствует либо добавляются
+    цифры
+    :param base_dn:
+    :param name:
+    :return:
+    """
+    if not path.exists(dn):
+        return dn
+    base_dn, name = path.split(dn)
+    for n in range(0, 9999):
+        dn = path.join(base_dn, "%s.%02d" % (name, n))
+        if not path.exists(dn):
+            return dn
+    raise FilesError(_("No directory name available for %s") % dn)
+
+
+def dir_sync(src, dst):
+    """
+    Скопировать файлы из src в dst, заменяя одинаковые файлы
+    :return:
+    """
+    if not path.isdir(src):
+        raise FilesError(_("%s is not directory") % src)
+    if path.lexists(dst):
+        dst_stat = os.lstat(dst)
+        if stat.S_ISDIR(dst_stat.st_mode):
+            for fn in listDirectory(src):
+                src_fn = path.join(src, fn)
+                dst_fn = path.join(dst, fn)
+                fn_stat = os.lstat(src_fn)
+                if stat.S_ISDIR(fn_stat.st_mode):
+                    dir_sync(src_fn, dst_fn)
+                    continue
+                if path.lexists(dst_fn):
+                    dst_stat = os.lstat(dst_fn)
+                    if stat.S_ISDIR(dst_stat.st_mode):
+                        shutil.rmtree(dst_fn)
+                    else:
+                        os.unlink(dst_fn)
+                shutil.copy2(src_fn, dst_fn)
+            return
+        else:
+            os.unlink(dst)
+    shutil.copytree(src, dst)
+
+
+def tar_directory(dn, archfile):
+    """
+    Заархивировать указанную директорию
+    :param dn:
+    :param archfile:
+    :return:
+    """
+    try:
+        with tarfile.open(archfile, "w:bz2") as tar:
+            for x in listDirectory(dn, fullPath=True):
+                tar.add(x, path.basename(x))
+    except (IOError, OSError) as e:
+        raise FilesError(_("Failed to create tarball: %s") % str(e))
